#include "grabcut.h"

#include <QPainter>
#include "../../core/functions.h"
#include "opencv2/imgproc.hpp"


GrabCut::GrabCut(QString framePath, QPolygonF polyArea):
    objectBondary(polyArea),
    image(cv::imread(framePath.toUtf8().constData(), CV_LOAD_IMAGE_COLOR))
{
}

void GrabCut::segmentImage(int iteration, int method)
{
   Q_UNUSED(method);
   cv::Mat bgdModel, fgdModel;
   cv::Rect rect(round(objectBondary.boundingRect().x()),round(objectBondary.boundingRect().y()),round(objectBondary.boundingRect().width()),round(objectBondary.boundingRect().height()));

   //cv::Rect rect(0,0,image.cols,image.rows);

   cv::Mat mask;
   mask.create(image.size(),CV_8UC1);
   mask.setTo( cv::GC_BGD );

   //(mask(rect)).setTo(cv::Scalar(cv::GC_PR_FGD) );

   std::vector<cv::Point> tmp = Functions::getOpenCvPolygon(objectBondary);
   const cv::Point* firstPolyPoint[1] = { &tmp[0] };
   int numberOfPoints = tmp.size();
   cv::fillPoly(mask, firstPolyPoint, &numberOfPoints, 1, cv::Scalar(cv::GC_FGD), 8);


   int thickness=round(qMax(objectBondary.boundingRect().width(),objectBondary.boundingRect().height())*0.20);
   if(thickness<1)
       thickness=1;

   cv::polylines(mask,firstPolyPoint,&numberOfPoints,1,true,cv::Scalar(cv::GC_PR_FGD),thickness,8,0);

   //for test only
   cv::Mat res;
   cv::Mat binMask;
   getBinMask(mask,binMask);
   image.copyTo(res,binMask);
   QString im;
   QDateTime now = QDateTime::currentDateTime();
   im="/work/seg/"+QString::number(now.toMSecsSinceEpoch())+".jpg";
   cv::imwrite(im.toStdString(), res);
   //for test only

   for(int i=1;i<=iteration;i++){
       grabCut(image,mask,rect,bgdModel, fgdModel,i,cv::GC_INIT_WITH_MASK);
   }

   //cv::Mat res;
   //cv::Mat binMask;
   getBinMask(mask,binMask);
   image.copyTo(res,binMask);
 //  QString im;
 //  QDateTime now = QDateTime::currentDateTime();
  // im="/work/seg/"+QString::number(now.toMSecsSinceEpoch())+".jpg";
  // cv::imwrite(im.toStdString(), res);

   std::vector< std::vector<cv::Point> > contours;
   cv::findContours(binMask, contours, CV_RETR_EXTERNAL, CV_CHAIN_APPROX_TC89_KCOS);

   std::vector<cv::Point> contour;
   if(contours.size()>1){
       int maxPoints = -1;
       for (std::vector< std::vector<cv::Point> >::iterator it = contours.begin() ; it != contours.end(); ++it){
           if((int)it->size()>maxPoints){
               maxPoints=it->size();
               contour=*it;
           }
       }
   }
   else
       contour=contours.at(0);

   objectBondary=Functions::getQPolygonF(contour);
}

const QPolygonF GrabCut::getObjectBondary()
{
    return objectBondary;
}

void GrabCut::getBinMask(const cv::Mat &comMask, cv::Mat &binMask)
{
    if( comMask.empty() || comMask.type()!=CV_8UC1 )
        qDebug()<<"comMask is empty or has incorrect type (not CV_8UC1)";
    if( binMask.empty() || binMask.rows!=comMask.rows || binMask.cols!=comMask.cols)
        binMask.create( comMask.size(), CV_8UC1);
    binMask = comMask & 1;
}
