#include "keyframegraphicsitem.h"

#include <QDebug>
#include <QMenu>
#include <QAction>
#include <QActionGroup>

#include "../globalobjects.h"

KeyFrameGraphicsItem::KeyFrameGraphicsItem(QGraphicsScene* s, QPointF p, KeyFrame *kf) :
    keyFrame(kf)
{

    setPos(p);
    loadStyle();

    s->addItem(this);

    connect(keyFrame, SIGNAL(hasChanged(int)), this, SLOT(reload()));
}

void KeyFrameGraphicsItem::reload()
{
    loadStyle();
}

void KeyFrameGraphicsItem::loadStyle()
{
    int radius=5;

    QColor c = this->keyFrame->object->color;

    setPen(QPen(Qt::black));
    if(keyFrame->type == KeyFrame::SetVisible) {
        setBrush(QBrush(c));
        radius = 5;
    } else if (keyFrame->type == KeyFrame::SetOccluded) {
        setBrush(QBrush(c, Qt::Dense3Pattern));
        radius = 5;
    } else if (keyFrame->type == KeyFrame::Automatic) {
        setPen(QPen(Qt::gray,1,Qt::SolidLine));
        QColor alpha = c;
        alpha.setAlpha(127);
        setBrush(QBrush(alpha));
        radius = 3;
    }
    else
        radius = 1;

    setRect(-1.414*radius, -1.414*radius, 2.824*radius, 2.824*radius);

    if(keyFrame->type == KeyFrame::NotSet) {
        hide();
    } else {
        show();
    }
}

void KeyFrameGraphicsItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
{
    QMenu menu(0);

    menu.addAction(this->keyFrame->object->name)->setEnabled(false);
    menu.addSeparator();

    QAction *removeAction = menu.addAction("Remove Keyframe");
    menu.addSeparator();
    QActionGroup typegroup(this);

    QAction *typeVisibleAction = menu.addAction("Visible");
    typeVisibleAction->setCheckable(true);
    typeVisibleAction->setActionGroup(&typegroup);
    typeVisibleAction->setChecked(keyFrame->type == KeyFrame::SetVisible);

    QAction *typeOccludedAction = menu.addAction("Occluded");
    typeOccludedAction->setCheckable(true);
    typeOccludedAction->setActionGroup(&typegroup);
    typeOccludedAction->setChecked(keyFrame->type == KeyFrame::SetOccluded);

    QAction *typeAutomaticAction = menu.addAction("Automatic");
    typeAutomaticAction->setCheckable(true);
    typeAutomaticAction->setActionGroup(&typegroup);
    typeAutomaticAction->setChecked(keyFrame->type == KeyFrame::Automatic);

    menu.addSeparator();

    QMenu *onTopMenu = menu.addMenu("switch order with");
    QList<QAction*> switchActions;
    QList<TrackedObject*> actionObjectMap;
    foreach(TrackedObject* obj, g_state->objectsList) {
        if(obj->id != this->keyFrame->object->id) {
            if(obj->keyFrames[this->keyFrame->frame]->type == KeyFrame::SetVisible) {
                QAction* tmp = onTopMenu->addAction(obj->name);
                switchActions.append(tmp);
                actionObjectMap.append(obj);
            }
        }
    }



    QAction *selectedAction = menu.exec(event->screenPos());
    if ( selectedAction != 0 ) {
        if(selectedAction == removeAction) {
            keyFrame->setNewPolygon(QPolygonF());
            keyFrame->setType(KeyFrame::NotSet);
        }
        else if(selectedAction == typeVisibleAction && keyFrame->type != KeyFrame::SetVisible) {
            keyFrame->setType(KeyFrame::SetVisible);
        }
        else if(selectedAction == typeOccludedAction && keyFrame->type != KeyFrame::SetOccluded) {
            keyFrame->setType(KeyFrame::SetOccluded);
        }
        else if(selectedAction == typeAutomaticAction && keyFrame->type != KeyFrame::Automatic) {
            keyFrame->setType(KeyFrame::Automatic);
        }
        else {
            for(int i = 0; i < switchActions.count(); ++i) {
                QAction* action = switchActions[i];
                if(selectedAction == action) {
                    for(int f = this->keyFrame->frame; f < g_state->vid->frameCount; ++f) {
                        int own = this->keyFrame->object->keyFrames[f]->zOrder;
                        int other = actionObjectMap[i]->keyFrames[f]->zOrder;
                        this->keyFrame->object->keyFrames[f]->zOrder = other;
                        actionObjectMap[i]->keyFrames[f]->zOrder = own;
                    }
                    emit this->keyFrame->object->hasChanged(TrackedObject::ZOrderChange);
                }
            }
        }

    }
}

